// -----------------------------------------------------------------------------
// DogsEar.Pas                Copyright (c) 1996-99 DithoSoft Software Solutions
// -----------------------------------------------------------------------------
// "Eselsohr"-Komponente: Kann in Anwendungen, die als Buch aufgebaut sind, zum
//                        Blttern verwendet werden.
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Properties: AlignDogsEar    - adLeft (linke Seite), adRight (rechte Seite)
//             ColorDogsEar    - Farbe der umgeklappten Ecke
//             ColorBackground - Farbe des Hintergrundes (Tranparenz simulieren)
//             ColorFrame      - Farbe des Rahmens
//             FrameStyle      - fsFrame (Rahmen), fsNone (kein Rahmen
//             MultiPage       - True: zwei Eselsohren (1 Seite, mehrere Seiten)
//                               False: ein Eselsohr (1 Seite)
//             NumPages        - Zahl der Seiten, die bei Multipage geblttert
//                               wird
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// Events:     OnClick         - Wenn angeklickt wird
// -----------------------------------------------------------------------------
unit DogsEar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TDogsEarAlign      = (adLeft, adRight);         // Ausrichtung des Eselsohres
  TDogsEarFrameStyle = (dfFrame, dfNoFrame);      // Rahmenart des Eselsohres
  TDogsEarNumPages   = 1..MaxInt;                 // Zahl der Scrollseiten
  TDogsEarNotify     = procedure (Sender : TObject;
                                 NumPages : TDogsEarNumPages) of Object;
  TDogsEarPoly = Array[0..3] of TPoint;



  TDogsEar = class(TGraphicControl)
  private
    FAlignDogsEar     : TDogsEarAlign;
    FColorBackground,
    FColorDogsEar,
    FColorFrame       : TColor;
    FFrameStyle       : TDogsEarFrameStyle;
    FMultiPage        : Boolean;
    FNumPages         : TDogsEarNumPages;
    FClickEvent       : TDogsEarNotify;
  private
    procedure SetAlign(Align : TDogsEarAlign);
    procedure SetColorBackground(Color : TColor);
    procedure SetColorDogsEar(Color : TColor);
    procedure SetColorFrame(Color : TColor);
    procedure SetFrameStyle(Style : TDogsEarFrameStyle);
    procedure SetMultiPage(MP : Boolean);
    procedure SetNumPages(Pages : TDogsEarNumPages);

  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
                        X, Y: Integer); override;
    function  CalcSizedPoly(Source : TDogsEarPoly) : TDogsEarPoly;

  public
    constructor Create(AOwner : TComponent); override;

  published
    property AlignDogsEar : TDogsEarAlign read FAlignDogsEar write SetAlign;
    property ColorBackground : TColor read FColorBackground write SetColorBackground;
    property ColorDogsEar : TColor read FColorDogsEar write SetColorDogsEar;
    property ColorFrame : TColor read FColorFrame write SetColorFrame;
    property Enabled;
    property FrameStyle : TDogsEarFrameStyle read FFrameStyle write SetFrameStyle;
    property MultiPage : Boolean read FMultiPage write SetMultiPage;
    property NumPages : TDogsEarNumPages read FNumPages write SetNumPages;
    property OnDogsEarClick : TDogsEarNotify read FClickEvent write FClickEvent;
    property Visible;
  end;

procedure Register;

implementation


const
 // SmallDogsEar hat 3/4 der Gre des LargeDogsEar
 LargeDogsEarLeft  : TDogsEarPoly = ((x:0;y:0),
                                     (x:4;y:0),
                                     (x:4;y:4),
                                     (x:0;y:0));
 SmallDogsEarLeft  : TDogsEarPoly = ((x:0;y:0),
                                     (x:3;y:1),
                                     (x:4;y:4),
                                     (x:0;y:0));

 LargeDogsEarRight : TDogsEarPoly = ((x:0;y:0),
                                     (x:4;y:0),
                                     (x:0;y:4),
                                     (x:0;y:0));
 SmallDogsEarRight : TDogsEarPoly = ((x:1;y:1),
                                     (x:4;y:0),
                                     (x:0;y:4),
                                     (x:1;y:1));


// -----------------------------------------------------------------------------
// Registrierung in der Komponentenpalette
// -----------------------------------------------------------------------------
procedure Register;
begin
  RegisterComponents('Andere', [TDogsEar]);
end;



// -----------------------------------------------------------------------------
// Erstellen der Class und setzen der Default-Werte
// -----------------------------------------------------------------------------
constructor TDogsEar.Create(AOwner : TComponent);
begin
 Inherited Create(AOwner);

 // Default-Werte setzen
 FAlignDogsEar     := adLeft;
 FColorBackground  := clBtnFace;
 FColorDogsEar     := clWindow;
 FColorFrame       := clWindowFrame;
 FFrameStyle       := dfFrame;
 FMultiPage        := False;
 FNumPages         := 1;

 Width             := 30;
 Height            := 30;
end;



// -----------------------------------------------------------------------------
// Setzen von Properties
// -----------------------------------------------------------------------------
procedure TDogsEar.SetAlign(Align : TDogsEarAlign);
begin
 FAlignDogsEar := Align;
 Invalidate;
end;

procedure TDogsEar.SetColorBackground(Color : TColor);
begin
 FColorBackground := Color;
 Invalidate;
end;

procedure TDogsEar.SetColorDogsEar(Color : TColor);
begin
 FColorDogsEar := Color;
 Invalidate;
end;

procedure TDogsEar.SetColorFrame(Color : TColor);
begin
 FColorFrame := Color;
 Invalidate;
end;

procedure TDogsEar.SetFrameStyle(Style : TDogsEarFrameStyle);
begin
 FFrameStyle := Style;
 Invalidate;
end;

procedure TDogsEar.SetMultiPage(MP : Boolean);
begin
 FMultiPage := MP;
 Invalidate;
end;

procedure TDogsEar.SetNumPages(Pages : TDogsEarNumPages);
begin
 FNumPages := Pages;
 Invalidate;
end;



// -----------------------------------------------------------------------------
// Zeichnen des Eselsohres
// -----------------------------------------------------------------------------
procedure TDogsEar.Paint;
var Poly,
    LargeDogsEar,
    SmallDogsEar  : TDogsEarPoly;
begin
 // DogsEar mit Hintergrundfarbe ausmalen
 Canvas.Brush.Color := FColorBackground;

 if FFrameStyle = dfNoFrame then
  Canvas.Pen.Color := FColorBackground
 else
  Canvas.Pen.Color := FColorFrame;

 Canvas.Rectangle(0,0,ClientWidth,ClientHeight);

 // DogsEar einzeichnen. Wenn MultiPage gesetzt, zuerst das groe, dann das
 // kleine wegen der berlagerung
 // Je nach Align die rechte oder die linke Polygonvorlage verwenden
 if FAlignDogsEar = adLeft then begin
  LargeDogsEar := LargeDogsEarLeft;
  SmallDogsEar := SmallDogsEarLeft;
 end else begin
  LargeDogsEar := LargeDogsEarRight;
  SmallDogsEar := SmallDogsEarRight;
 end;

 // Zuerst die Polygon-Koordinaten neu berechnen
 Poly := CalcSizedPoly(LargeDogsEar);

 // Farben setzen und zeichnen
 Canvas.Brush.Color := FColorDogsEar;
 Canvas.Pen.Color   := FColorFrame;
 Canvas.Polygon(Poly);

 // Im MultiPage-Modus auch noch das kleine zeichnen. Ablauf siehe oben
 if FMultiPage then begin
  Poly := CalcSizedPoly(SmallDogsEar);
  Canvas.Polygon(Poly);
 end;
end;

function TDogsEar.CalcSizedPoly(Source : TDogsEarPoly) : TDogsEarPoly;
begin
 Result[0].x := (Source[0].x*Pred(ClientWidth)) div 4;
 Result[0].y := (Source[0].y*Pred(ClientHeight)) div 4;
 Result[1].x := (Source[1].x*Pred(ClientWidth)) div 4;
 Result[1].y := (Source[1].y*Pred(ClientHeight)) div 4;
 Result[2].x := (Source[2].x*Pred(ClientWidth)) div 4;
 Result[2].y := (Source[2].y*Pred(ClientHeight)) div 4;
 Result[3].x := (Source[3].x*Pred(ClientWidth)) div 4;
 Result[3].y := (Source[3].y*Pred(ClientHeight)) div 4;
end;



// -----------------------------------------------------------------------------
// Mausabfrage je nach Klickposition
// -----------------------------------------------------------------------------
procedure TDogsEar.MouseDown(Button: TMouseButton; Shift: TShiftState;
                             X, Y: Integer);
var Large,
    Small    : TDogsEarPoly;
    LargeRGN,
    SmallRGN : HRGN;
begin
 // Nichts machen, wenn kein ClickEvent vorhanden
 if not Assigned(FClickEvent) then Exit;

 // Bereiche nach Gre des DogsEar berechnen
 if AlignDogsEar = adLeft then begin
  Large := CalcSizedPoly(LargeDogsEarLeft);
  Small := CalcSizedPoly(SmallDogsEarLeft);
 end else begin
  Large := CalcSizedPoly(LargeDogsEarRight);
  Small := CalcSizedPoly(SmallDogsEarRight);
 end;

 // Regionen erstellen, damit man herausfinden kann, wo geklickt wurde
 LargeRGN := CreatePolygonRgn(Large,4,WINDING);
 SmallRGN := CreatePolygonRgn(Small,4,WINDING);

 // Wenn beide Regionen erstellt wurden, nachprfen, in welche geklickt wurde
 if (LargeRGN <> 0) and (SmallRGN <> 0) then begin
  // Bei MultiPage mu geprft werden
  if MultiPage then begin
   // ob in das kleine Feld geklickt wurde -> NumPages Seiten bergeben
   if (PtInRegion(SmallRGN,x,y)) then
    FClickEvent(Self,NumPages)
   else if (PtInRegion(LargeRGN,x,y)) then
   // oder nur in das groe -> 1 Seite bergeben
    FClickEvent(Self,1);
  end else begin
   // Das entfllt, wenn nicht MultiPage
   if (PtInRegion(LargeRGN,x,y)) then
    FClickEvent(Self,1);
  end;
 end;

 // Aufrumen
 if LargeRGN <> 0 then DeleteObject(LargeRGN);
 if SmallRGN <> 0 then DeleteObject(SmallRGN);
end;


end.
